@@ -53,6 +53,7 @@ pub enum casper_error_t {
     CASPER_FFI_PTR_NULL_BUT_REQUIRED = -22,
     CASPER_CONFLICTING_ARGUMENTS = -23,
     CASPER_DEPLOY_SIZE_TOO_LARGE = -24,
+    CASPER_FAILED_TO_CREATE_DICTIONARY_IDENTIFIER = -25,
 }
 
 trait AsFFIError {
@@ -86,6 +87,9 @@ impl AsFFIError for Error {
             Error::FFIPtrNullButRequired(_) => casper_error_t::CASPER_FFI_PTR_NULL_BUT_REQUIRED,
             Error::ConflictingArguments { .. } => casper_error_t::CASPER_CONFLICTING_ARGUMENTS,
             Error::DeploySizeTooLarge(_) => casper_error_t::CASPER_DEPLOY_SIZE_TOO_LARGE,
+            Error::FailedToParseDictionaryIdentifier => {
+                casper_error_t::CASPER_FAILED_TO_CREATE_DICTIONARY_IDENTIFIER
+            }
         }
     }
 }
@@ -295,6 +299,7 @@ pub extern "C" fn casper_make_deploy(
     deploy_params: *const casper_deploy_params_t,
     session_params: *const casper_session_params_t,
     payment_params: *const casper_payment_params_t,
+    force: bool,
 ) -> casper_error_t {
     let maybe_output_path = try_unsafe_arg!(maybe_output_path);
     let deploy_params = try_arg_into!(deploy_params);
@@ -305,6 +310,7 @@ pub extern "C" fn casper_make_deploy(
         deploy_params,
         session_params,
         payment_params,
+        force,
     );
     try_unwrap_result!(result);
     casper_error_t::CASPER_SUCCESS
@@ -319,11 +325,12 @@ pub extern "C" fn casper_sign_deploy_file(
     input_path: *const c_char,
     secret_key: *const c_char,
     maybe_output_path: *const c_char,
+    force: bool,
 ) -> casper_error_t {
     let input_path = try_unsafe_arg!(input_path);
     let secret_key = try_unsafe_arg!(secret_key);
     let maybe_output_path = try_unsafe_arg!(maybe_output_path);
-    let result = super::sign_deploy_file(input_path, secret_key, maybe_output_path);
+    let result = super::sign_deploy_file(input_path, secret_key, maybe_output_path, force);
     try_unwrap_result!(result);
     casper_error_t::CASPER_SUCCESS
 }
@@ -407,6 +414,7 @@ pub extern "C" fn casper_make_transfer(
     transfer_id: *const c_char,
     deploy_params: *const casper_deploy_params_t,
     payment_params: *const casper_payment_params_t,
+    force: bool,
 ) -> casper_error_t {
     let maybe_output_path = try_unsafe_arg!(maybe_output_path);
     let amount = try_unsafe_arg!(amount);
@@ -421,6 +429,7 @@ pub extern "C" fn casper_make_transfer(
         transfer_id,
         deploy_params,
         payment_params,
+        force,
     );
     try_unwrap_result!(result);
     casper_error_t::CASPER_SUCCESS
@@ -634,6 +643,7 @@ pub extern "C" fn casper_get_era_info_by_switch_block(
 pub extern "C" fn casper_get_auction_info(
     maybe_rpc_id: *const c_char,
     node_address: *const c_char,
+    maybe_block_id: *const c_char,
     verbosity_level: u64,
     response_buf: *mut c_uchar,
     response_buf_len: usize,
@@ -642,8 +652,10 @@ pub extern "C" fn casper_get_auction_info(
     let runtime = try_unwrap_option!(&mut *runtime, or_else => Error::FFISetupNotCalled);
     let maybe_rpc_id = try_unsafe_arg!(maybe_rpc_id);
     let node_address = try_unsafe_arg!(node_address);
+    let maybe_block_id = try_unsafe_arg!(maybe_block_id);
     runtime.block_on(async move {
-        let result = super::get_auction_info(maybe_rpc_id, node_address, verbosity_level);
+        let result =
+            super::get_auction_info(maybe_rpc_id, node_address, verbosity_level, maybe_block_id);
         let response = try_unwrap_rpc!(result);
         copy_str_to_buf(&response, response_buf, response_buf_len);
         casper_error_t::CASPER_SUCCESS
@@ -782,6 +794,7 @@ pub struct casper_session_params_t {
     session_args_complex: *const c_char,
     session_version: *const c_char,
     session_entry_point: *const c_char,
+    is_session_transfer: bool,
 }
 
 impl TryInto<super::SessionStrParams<'static>> for casper_session_params_t {
@@ -829,6 +842,7 @@ impl TryInto<super::SessionStrParams<'static>> for casper_session_params_t {
             session_args_complex,
             session_version,
             session_entry_point,
+            is_session_transfer: self.is_session_transfer,
         })
     }
 }
