@@ -1,8 +1,8 @@
 //! # Casper node client library
 #![doc(
     html_root_url = "https://docs.rs/casper-client/0.1.0",
-    html_favicon_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Favicon_RGB_50px.png",
-    html_logo_url = "https://raw.githubusercontent.com/CasperLabs/casper-node/master/images/CasperLabs_Logo_Symbol_RGB.png",
+    html_favicon_url = "https://raw.githubusercontent.com/casper-network/casper-node/master/images/CasperLabs_Logo_Favicon_RGB_50px.png",
+    html_logo_url = "https://raw.githubusercontent.com/casper-network/casper-node/master/images/CasperLabs_Logo_Symbol_RGB.png",
     test(attr(forbid(warnings)))
 )]
 #![warn(
@@ -28,15 +28,14 @@ use jsonrpc_lite::JsonRpc;
 use serde::Serialize;
 
 use casper_execution_engine::core::engine_state::ExecutableDeployItem;
-use casper_node::types::Deploy;
-use casper_types::{UIntParseError, U512};
+use casper_node::{rpcs::state::DictionaryIdentifier, types::Deploy};
+use casper_types::{Key, UIntParseError, U512};
 
 pub use cl_type::help;
 pub use deploy::ListDeploysResult;
-use deploy::{DeployExt, DeployParams};
+use deploy::{DeployExt, DeployParams, OutputKind};
 pub use error::Error;
 use error::Result;
-use parsing::none_if_empty;
 use rpc::{RpcCall, TransferTarget};
 pub use validation::ValidateResponseError;
 
@@ -80,36 +79,38 @@ pub fn put_deploy(
 /// [`sign_deploy_file()`](fn.sign_deploy_file.html) and then sent to the network for execution
 /// using [`send_deploy_file()`](fn.send_deploy_file.html).
 ///
-/// * `maybe_output_path` specifies the output file, or if empty, will print it to `stdout`. If the
-///   file already exists, it will be overwritten.
+/// * `maybe_output_path` specifies the output file, or if empty, will print it to `stdout`.
 /// * `deploy_params` contains deploy-related options for this `Deploy`. See
 ///   [`DeployStrParams`](struct.DeployStrParams.html) for more details.
 /// * `session_params` contains session-related options for this `Deploy`. See
 ///   [`SessionStrParams`](struct.SessionStrParams.html) for more details.
 /// * `payment_params` contains payment-related options for this `Deploy`. See
 ///   [`PaymentStrParams`](struct.PaymentStrParams.html) for more details.
+/// * If `force` is true, and a file exists at `maybe_output_path`, it will be overwritten. If
+///   `force` is false and a file exists at `maybe_output_path`,
+///   [`Error::FileAlreadyExists`](enum.Error.html#variant.FileAlreadyExists) is returned and a file
+///   will not be written.
 pub fn make_deploy(
     maybe_output_path: &str,
     deploy_params: DeployStrParams<'_>,
     session_params: SessionStrParams<'_>,
     payment_params: PaymentStrParams<'_>,
+    force: bool,
 ) -> Result<()> {
-    let output = deploy::output_or_stdout(none_if_empty(maybe_output_path)).map_err(|error| {
-        Error::IoError {
-            context: format!(
-                "unable to get file or stdout, provided '{:?}'",
-                maybe_output_path
-            ),
-            error,
-        }
-    })?;
+    let output = if maybe_output_path.is_empty() {
+        OutputKind::Stdout
+    } else {
+        OutputKind::file(maybe_output_path, force)
+    };
 
     Deploy::with_payment_and_session(
         deploy_params.try_into()?,
         payment_params.try_into()?,
         session_params.try_into()?,
     )?
-    .write_deploy(output)
+    .write_deploy(output.get()?)?;
+
+    output.commit()
 }
 
 /// Reads a previously-saved `Deploy` from a file, cryptographically signs it, and outputs it to a
@@ -117,26 +118,33 @@ pub fn make_deploy(
 ///
 /// * `input_path` specifies the path to the previously-saved `Deploy` file.
 /// * `secret_key` specifies the path to the secret key with which to sign the `Deploy`.
-/// * `maybe_output_path` specifies the output file, or if empty, will print it to `stdout`. If the
-///   file already exists, it will be overwritten.
-pub fn sign_deploy_file(input_path: &str, secret_key: &str, maybe_output_path: &str) -> Result<()> {
+/// * `maybe_output_path` specifies the output file, or if empty, will print it to `stdout`.
+/// * If `force` is true, and a file exists at `maybe_output_path`, it will be overwritten. If
+///   `force` is false and a file exists at `maybe_output_path`,
+///   [`Error::FileAlreadyExists`](enum.Error.html#variant.FileAlreadyExists) is returned and a file
+///   will not be written.
+pub fn sign_deploy_file(
+    input_path: &str,
+    secret_key: &str,
+    maybe_output_path: &str,
+    force: bool,
+) -> Result<()> {
     let secret_key = parsing::secret_key(secret_key)?;
-    let maybe_output_path = none_if_empty(maybe_output_path);
 
     let input = fs::read(input_path).map_err(|error| Error::IoError {
         context: format!("unable to read deploy file at '{}'", input_path),
         error,
     })?;
 
-    let output = deploy::output_or_stdout(maybe_output_path).map_err(|error| Error::IoError {
-        context: format!(
-            "unable to get file or stdout, provided '{:?}'",
-            maybe_output_path
-        ),
-        error,
-    })?;
+    let output = if maybe_output_path.is_empty() {
+        OutputKind::Stdout
+    } else {
+        OutputKind::file(maybe_output_path, force)
+    };
 
-    Deploy::sign_and_write_deploy(Cursor::new(input), secret_key, output)
+    Deploy::sign_and_write_deploy(Cursor::new(input), secret_key, output.get()?)?;
+
+    output.commit()
 }
 
 /// Reads a previously-saved `Deploy` from a file and sends it to the network for execution.
@@ -215,8 +223,7 @@ pub fn transfer(
 /// [`sign_deploy_file()`](fn.sign_deploy_file.html) and then sent to the network for execution
 /// using [`send_deploy_file()`](fn.send_deploy_file.html).
 ///
-/// * `maybe_output_path` specifies the output file, or if empty, will print it to `stdout`. If the
-///   file already exists, it will be overwritten.
+/// * `maybe_output_path` specifies the output file, or if empty, will print it to `stdout`.
 /// * `amount` is a string to be parsed as a `U512` specifying the amount to be transferred.
 /// * `target_account` is the account `PublicKey` into which the funds will be transferred,
 ///   formatted as a hex-encoded string. The account's main purse will receive the funds.
@@ -226,6 +233,10 @@ pub fn transfer(
 ///   [`DeployStrParams`](struct.DeployStrParams.html) for more details.
 /// * `payment_params` contains payment-related options for this `Deploy`. See
 ///   [`PaymentStrParams`](struct.PaymentStrParams.html) for more details.
+/// * If `force` is true, and a file exists at `maybe_output_path`, it will be overwritten. If
+///   `force` is false and a file exists at `maybe_output_path`,
+///   [`Error::FileAlreadyExists`](enum.Error.html#variant.FileAlreadyExists) is returned and a file
+///   will not be written.
 pub fn make_transfer(
     maybe_output_path: &str,
     amount: &str,
@@ -233,6 +244,7 @@ pub fn make_transfer(
     transfer_id: &str,
     deploy_params: DeployStrParams<'_>,
     payment_params: PaymentStrParams<'_>,
+    force: bool,
 ) -> Result<()> {
     let amount = U512::from_dec_str(amount)
         .map_err(|err| Error::FailedToParseUint("amount", UIntParseError::FromDecStr(err)))?;
@@ -240,15 +252,11 @@ pub fn make_transfer(
     let target = parsing::get_transfer_target(target_account)?;
     let transfer_id = parsing::transfer_id(transfer_id)?;
 
-    let output = deploy::output_or_stdout(none_if_empty(maybe_output_path)).map_err(|error| {
-        Error::IoError {
-            context: format!(
-                "unable to get file or stdout, provided '{:?}'",
-                maybe_output_path
-            ),
-            error,
-        }
-    })?;
+    let output = if maybe_output_path.is_empty() {
+        OutputKind::Stdout
+    } else {
+        OutputKind::file(maybe_output_path, force)
+    };
 
     Deploy::new_transfer(
         amount,
@@ -258,7 +266,9 @@ pub fn make_transfer(
         deploy_params.try_into()?,
         payment_params.try_into()?,
     )?
-    .write_deploy(output)
+    .write_deploy(output.get()?)?;
+
+    output.commit()
 }
 
 /// Retrieves a `Deploy` from the network.
@@ -452,12 +462,42 @@ pub fn get_era_info_by_switch_block(
 ///   count of the field.  When `verbosity_level` is greater than `1`, the request will be printed
 ///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
 ///   will not be printed to `stdout`.
+/// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
+///   `Block` height or empty. If empty, era information from the latest block will be returned if
+///   available.
 pub fn get_auction_info(
     maybe_rpc_id: &str,
     node_address: &str,
     verbosity_level: u64,
+    maybe_block_id: &str,
 ) -> Result<JsonRpc> {
-    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_auction_info()
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level).get_auction_info(maybe_block_id)
+}
+
+/// Retrieves an Account from the network.
+///
+/// * `maybe_rpc_id` is the JSON-RPC identifier, applied to the request and returned in the
+///   response. If it can be parsed as an `i64` it will be used as a JSON integer. If empty, a
+///   random `i64` will be assigned. Otherwise the provided string will be used verbatim.
+/// * `node_address` is the hostname or IP and port of the node on which the HTTP service is
+///   running, e.g. `"http://127.0.0.1:7777"`.
+/// * When `verbosity_level` is `1`, the JSON-RPC request will be printed to `stdout` with long
+///   string fields (e.g. hex-formatted raw Wasm bytes) shortened to a string indicating the char
+///   count of the field.  When `verbosity_level` is greater than `1`, the request will be printed
+///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
+///   will not be printed to `stdout`.
+/// * `public_key` the public key associated with the `Account`
+/// * `maybe_block_id` must be a hex-encoded, 32-byte hash digest or a `u64` representing the
+///   `Block` height or empty. If empty, the latest `Block` will be retrieved.
+pub fn get_account_info(
+    maybe_rpc_id: &str,
+    node_address: &str,
+    verbosity_level: u64,
+    public_key: &str,
+    maybe_block_id: &str,
+) -> Result<JsonRpc> {
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_account_info(public_key, maybe_block_id)
 }
 
 /// Retrieves information and examples for all currently supported RPCs.
@@ -476,6 +516,31 @@ pub fn list_rpcs(maybe_rpc_id: &str, node_address: &str, verbosity_level: u64) -
     RpcCall::new(maybe_rpc_id, node_address, verbosity_level).list_rpcs()
 }
 
+/// Retrieves a stored value from the network.
+///
+/// * `maybe_rpc_id` is the JSON-RPC identifier, applied to the request and returned in the
+///   response. If it can be parsed as an `i64` it will be used as a JSON integer. If empty, a
+///   random `i64` will be assigned. Otherwise the provided string will be used verbatim.
+/// * `node_address` is the hostname or IP and port of the node on which the HTTP service is
+///   running, e.g. `"http://127.0.0.1:7777"`.
+/// * When `verbosity_level` is `1`, the JSON-RPC request will be printed to `stdout` with long
+///   string fields (e.g. hex-formatted raw Wasm bytes) shortened to a string indicating the char
+///   count of the field.  When `verbosity_level` is greater than `1`, the request will be printed
+///   to `stdout` with no abbreviation of long fields.  When `verbosity_level` is `0`, the request
+///   will not be printed to `stdout`.
+/// * `state_root_hash` must be a hex-encoded, 32-byte hash digest.
+/// * `dictionary_str_params` contains options to query a dictionary item.
+pub fn get_dictionary(
+    maybe_rpc_id: &str,
+    node_address: &str,
+    verbosity_level: u64,
+    state_root_hash: &str,
+    dictionary_str_params: DictionaryItemStrParams<'_>,
+) -> Result<JsonRpc> {
+    RpcCall::new(maybe_rpc_id, node_address, verbosity_level)
+        .get_dictionary_item(state_root_hash, dictionary_str_params)
+}
+
 /// Container for `Deploy` construction options.
 #[derive(Default, Debug)]
 pub struct DeployStrParams<'a> {
@@ -782,6 +847,7 @@ impl<'a> TryInto<ExecutableDeployItem> for SessionStrParams<'a> {
             session_args_complex,
             session_version,
             session_entry_point,
+            is_session_transfer,
         } = self;
 
         parsing::parse_session_info(
@@ -794,6 +860,7 @@ impl<'a> TryInto<ExecutableDeployItem> for SessionStrParams<'a> {
             session_args_complex,
             session_version,
             session_entry_point,
+            is_session_transfer,
         )
     }
 }
@@ -829,6 +896,7 @@ pub struct SessionStrParams<'a> {
     session_args_complex: &'a str,
     session_version: &'a str,
     session_entry_point: &'a str,
+    is_session_transfer: bool,
 }
 
 impl<'a> SessionStrParams<'a> {
@@ -947,6 +1015,103 @@ impl<'a> SessionStrParams<'a> {
             ..Default::default()
         }
     }
+
+    /// Constructs a `SessionStrParams` representing a `Transfer` type of `Deploy`.
+    ///
+    /// * See the struct docs for a description of [`session_args_simple`](#session_args_simple) and
+    ///   [`session_args_complex`](#session_args_complex).
+    pub fn with_transfer(session_args_simple: Vec<&'a str>, session_args_complex: &'a str) -> Self {
+        Self {
+            is_session_transfer: true,
+            session_args_simple,
+            session_args_complex,
+            ..Default::default()
+        }
+    }
+}
+
+/// Various ways of uniquely identifying a dictionary entry.
+pub enum DictionaryItemStrParams<'a> {
+    /// Lookup a dictionary item via an Account's named keys.
+    AccountNamedKey {
+        /// The account key as a formatted string whose named keys contains dictionary_name.
+        key: &'a str,
+        /// The named key under which the dictionary seed URef is stored.
+        dictionary_name: &'a str,
+        /// The dictionary item key formatted as a string.
+        dictionary_item_key: &'a str,
+    },
+    /// Lookup a dictionary item via a Contract's named keys.
+    ContractNamedKey {
+        /// The contract key as a formatted string whose named keys contains dictionary_name.
+        key: &'a str,
+        /// The named key under which the dictionary seed URef is stored.
+        dictionary_name: &'a str,
+        /// The dictionary item key formatted as a string.
+        dictionary_item_key: &'a str,
+    },
+    /// Lookup a dictionary item via its seed URef.
+    URef {
+        /// The dictionary's seed URef.
+        seed_uref: &'a str,
+        /// The dictionary item key formatted as a string.
+        dictionary_item_key: &'a str,
+    },
+    /// Lookup a dictionary item via its unique key.
+    Dictionary(&'a str),
+}
+
+impl<'a> TryInto<DictionaryIdentifier> for DictionaryItemStrParams<'a> {
+    type Error = Error;
+
+    fn try_into(self) -> Result<DictionaryIdentifier> {
+        match self {
+            DictionaryItemStrParams::AccountNamedKey {
+                key,
+                dictionary_item_key,
+                dictionary_name,
+            } => {
+                let key = Key::from_formatted_str(key)
+                    .map_err(|_| Error::FailedToParseDictionaryIdentifier)?;
+                Ok(DictionaryIdentifier::AccountNamedKey {
+                    key: key.to_formatted_string(),
+                    dictionary_name: dictionary_name.to_string(),
+                    dictionary_item_key: dictionary_item_key.to_string(),
+                })
+            }
+            DictionaryItemStrParams::ContractNamedKey {
+                key,
+                dictionary_item_key,
+                dictionary_name,
+            } => {
+                let key = Key::from_formatted_str(key)
+                    .map_err(|_| Error::FailedToParseDictionaryIdentifier)?;
+                Ok(DictionaryIdentifier::ContractNamedKey {
+                    key: key.to_formatted_string(),
+                    dictionary_name: dictionary_name.to_string(),
+                    dictionary_item_key: dictionary_item_key.to_string(),
+                })
+            }
+            DictionaryItemStrParams::URef {
+                seed_uref,
+                dictionary_item_key,
+            } => {
+                let uref = Key::from_formatted_str(seed_uref)
+                    .map_err(|_| Error::FailedToParseDictionaryIdentifier)?;
+                Ok(DictionaryIdentifier::URef {
+                    seed_uref: uref.to_formatted_string(),
+                    dictionary_item_key: dictionary_item_key.to_string(),
+                })
+            }
+            DictionaryItemStrParams::Dictionary(dictionary_key) => {
+                let dictionary_key = Key::from_formatted_str(dictionary_key)
+                    .map_err(|_| Error::FailedToParseDictionaryIdentifier)?;
+                Ok(DictionaryIdentifier::Dictionary(
+                    dictionary_key.to_formatted_string(),
+                ))
+            }
+        }
+    }
 }
 
 /// When `verbosity_level` is `1`, the value will be printed to `stdout` with long string fields
